import copy
import six

from datetime import datetime
from oslo_utils import uuidutils


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""


def generate_request_id():
    return 'req-' + uuidutils.generate_uuid()


class RequestContext(object):
    """Security context and request information.

    Represents the user taking a given action within the system.

    """

    def __init__(self, user_id, project_id, is_admin=None, read_deleted="no",
                 roles=None, remote_address=None, timestamp=None,
                 request_id=None, auth_token=None, overwrite=True,
                 quota_class=None, user_name=None, project_name=None,
                 service_catalog=None, instance_lock_checked=False, **kwargs):
        self.user_id = user_id
        self.project_id = project_id
        self.roles = roles or []
        self.read_deleted = read_deleted
        self.remote_address = remote_address
        if not timestamp:
            timestamp = datetime.utcnow()
        if isinstance(timestamp, six.string_types):
            timestamp = datetime.strptime(timestamp, '%Y-%m-%dT%H:%M:%S.%f')
        self.timestamp = timestamp
        if not request_id:
            request_id = generate_request_id()
        self.request_id = request_id
        self.auth_token = auth_token

        if service_catalog:
            # Only include required parts of service_catalog
            self.service_catalog = [
                s for s in service_catalog if s.get('type') in ('volume',)]
        else:
            # if list is empty or none
            self.service_catalog = []

        self.instance_lock_checked = instance_lock_checked

        # NOTE(markmc): this attribute is currently only used by the
        # rs_limits turnstile pre-processor.
        # See https://lists.launchpad.net/openstack/msg12200.html
        self.quota_class = quota_class
        self.user_name = user_name
        self.project_name = project_name
        self.is_admin = is_admin
        # if self.is_admin is None:
        #    self.is_admin = policy.check_is_admin(self)

        """
        if overwrite or not hasattr(local.store, 'context'):
            self.update_store()
        """
    def _get_read_deleted(self):
        return self._read_deleted

    def _set_read_deleted(self, read_deleted):
        if read_deleted not in ('no', 'yes', 'only'):
            raise ValueError("read_deleted can only be one of 'no', "
                             "'yes' or 'only', not %r" % read_deleted)
        self._read_deleted = read_deleted

    def _del_read_deleted(self):
        del self._read_deleted

    read_deleted = property(_get_read_deleted, _set_read_deleted,
                            _del_read_deleted)

    def update_store(self):
        # local.store.context = self
        pass

    def toDict(self):
        return {'user_id': self.user_id,
                'project_id': self.project_id,
                'is_admin': self.is_admin,
                'read_deleted': self.read_deleted,
                'roles': self.roles,
                'remote_address': self.remote_address,
                'timestamp': datetime.strptime(self.timestamp,
                                               '%Y-%m-%dT%H:%M:%S.%f'),
                'request_id': self.request_id,
                'auth_token': self.auth_token,
                'quota_class': self.quota_class,
                'user_name': self.user_name,
                'service_catalog': self.service_catalog,
                'project_name': self.project_name,
                'instance_lock_checked': self.instance_lock_checked,
                'tenant': self.tenant,
                'user': self.user}

    @classmethod
    def fromDict(cls, values):
        values.pop('user', None)
        values.pop('tenant', None)

        return cls(**values)

    def elevated(self, read_deleted=None):
        """Return a version of this context with admin flag set."""
        context = copy.copy(self)
        context.is_admin = True

        if 'admin' not in context.roles:
            context.roles.append('admin')

        if read_deleted is not None:
            context.read_deleted = read_deleted

        return context

    # NOTE(sirp): the openstack/common version of RequestContext uses
    # tenant/user whereas the Nova version uses project_id/user_id. We need
    # this shim in order to use context-aware code from openstack/common, like
    # logging, until we make the switch to using openstack/common's version of
    # RequestContext.
    @property
    def tenant(self):
        return self.project_id

    @property
    def user(self):
        return self.user_id


def get_admin_context(read_deleted="no"):
    return RequestContext(user_id=None,
                          project_id=None,
                          is_admin=True,
                          read_deleted=read_deleted,
                          overwrite=False)


def is_user_context(context):
    """Indicates if the request context is a normal user."""
    if not context:
        return False
    if context.is_admin:
        return False
    if not context.user_id or not context.project_id:
        return False
    return True



# coding: utf-8
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Test the ManagerCommand class.

"""

import mock

from argparse import ArgumentParser
from argparse import Namespace
from synergy.client.command import HTTPCommand
from synergy.client.command import ManagerCommand
from synergy.tests import base


class TestManagerCommand(base.TestCase):

    def setUp(self):
        super(TestManagerCommand, self).setUp()
        self.manager_command = ManagerCommand()

    def test_config_parser(self):
        """Check that the manager parser is set well."""
        root_parser = ArgumentParser()
        subs = root_parser.add_subparsers(dest="command_name")
        self.manager_command.configureParser(subs)

        # manager list
        res = root_parser.parse_args(["manager", "list"])
        ns = Namespace(command_name="manager", command="list")
        self.assertEqual(ns, res)

        self.assertRaises(
            SystemExit,
            root_parser.parse_args,
            ["manager", "list", "wrong-argument"])

        # manager status
        res = root_parser.parse_args(["manager", "status"])
        ns = Namespace(command_name="manager", command="status", manager=None)
        self.assertEqual(ns, res)

        res = root_parser.parse_args(["manager", "status", "TestManager"])
        ns = Namespace(
            command_name="manager",
            command="status",
            manager="TestManager")
        self.assertEqual(ns, res)

        res = root_parser.parse_args(
            ["manager", "status", "Test1"])
        ns = Namespace(
            command_name="manager",
            command="status",
            manager="Test1")
        self.assertEqual(ns, res)

        # manager start
        self.assertRaises(
            SystemExit,
            root_parser.parse_args,
            ["manager", "start"])

        res = root_parser.parse_args(["manager", "start", "TestManager"])
        ns = Namespace(
            command_name="manager",
            command="start",
            manager="TestManager")
        self.assertEqual(ns, res)

        res = root_parser.parse_args(
            ["manager", "start", "Test1"])
        ns = Namespace(
            command_name="manager",
            command="start",
            manager="Test1")
        self.assertEqual(ns, res)

        # manager stop
        self.assertRaises(
            SystemExit,
            root_parser.parse_args,
            ["manager", "stop"])

        res = root_parser.parse_args(["manager", "stop", "TestManager"])
        ns = Namespace(
            command_name="manager",
            command="stop",
            manager="TestManager")
        self.assertEqual(ns, res)

        res = root_parser.parse_args(
            ["manager", "stop", "Test1"])
        ns = Namespace(
            command_name="manager",
            command="stop",
            manager="Test1")
        self.assertEqual(ns, res)

    @mock.patch('synergy.client.command.tabulate')
    def test_execute_list(self, mock_tabulate):
        """Check the CLI output of "manager list"."""
        mock_parser = mock.Mock()
        mock_parser.args.command = "list"

        manager_a = mock.Mock()
        manager_a.getName.return_value = "ManagerA"
        manager_b = mock.Mock()
        manager_b.getName.return_value = "ManagerB"
        mgrs = [manager_a, manager_b]

        with mock.patch.object(HTTPCommand, 'execute', return_value=mgrs) as m:
            self.manager_command.execute(synergy_url="", args=mock_parser.args)

        m.assert_called_once_with("/synergy/list")

        headers = ["manager"]
        table = [["ManagerA"], ["ManagerB"]]
        mock_tabulate.assert_called_once_with(
            table,
            headers,
            tablefmt="fancy_grid")

    @mock.patch('synergy.client.command.tabulate')
    def test_execute_status_no_manager(self, mock_tabulate):
        """Check the CLI output of "manager status"."""
        # Mock the parser call
        mock_parser = mock.Mock()
        mock_parser.args.command = "status"
        mock_parser.args.manager = []

        # Mock 2 managers and their statuses
        manager_a = mock.Mock()
        manager_a.getName.return_value = "ManagerA"
        manager_a.getStatus.return_value = "UP"
        manager_a.getRate.return_value = 5
        manager_b = mock.Mock()
        manager_b.getName.return_value = "ManagerB"
        manager_b.getStatus.return_value = "DOWN"
        manager_b.getRate.return_value = 10
        mgrs = [manager_a, manager_b]

        # Execute "manager status"
        with mock.patch.object(HTTPCommand, 'execute', return_value=mgrs) as m:
            self.manager_command.execute(synergy_url="", args=mock_parser.args)

        # Check the executed call when we did "manager status"
        m.assert_called_once_with(
            "/synergy/status",
            {"manager": []})

        # Check the data when we call tabulate
        headers = ["manager", "status", "rate (min)"]
        table = [
            ["ManagerA", "UP", 5],
            ["ManagerB", "DOWN", 10]]
        mock_tabulate.assert_called_once_with(
            table,
            headers,
            tablefmt="fancy_grid")

    @mock.patch('synergy.client.command.tabulate')
    def test_execute_status_one_manager(self, mock_tabulate):
        """Check the CLI output of "manager status ManagerB"."""
        # Mock the parser call
        mock_parser = mock.Mock()
        mock_parser.args.command = "status"
        mock_parser.args.manager = ["ManagerB"]

        # Mock a manager and its status
        manager_b = mock.Mock()
        manager_b.getName.return_value = "ManagerB"
        manager_b.getStatus.return_value = "DOWN"
        manager_b.getRate.return_value = 10
        mgrs = [manager_b]

        # Execute "manager status ManagerA"
        with mock.patch.object(HTTPCommand, 'execute', return_value=mgrs) as m:
            self.manager_command.execute(synergy_url="", args=mock_parser.args)

        # Check the executed call when we did "manager status"
        m.assert_called_once_with(
            "/synergy/status",
            {"manager": ["ManagerB"]})

        # Check the data when we call tabulate
        headers = ["manager", "status", "rate (min)"]
        table = [["ManagerB", "DOWN", 10]]
        mock_tabulate.assert_called_once_with(
            table,
            headers,
            tablefmt="fancy_grid")

    @mock.patch('synergy.client.command.tabulate')
    def test_execute_start_manager(self, mock_tabulate):
        """Check the CLI output of "manager start ManagerA"."""
        # Mock the parser call
        mock_parser = mock.Mock()
        mock_parser.args.command = "start"
        mock_parser.args.manager = "ManagerA"

        # Mock a manager
        manager_a = mock.Mock()
        manager_a.getName.return_value = "ManagerA"
        manager_a.getStatus.return_value = "RUNNING"
        manager_a.get.return_value = "started successfully"
        manager_a.getRate.return_value = 1
        mgrs = manager_a

        # Execute "manager start ManagerA"
        with mock.patch.object(HTTPCommand, 'execute', return_value=mgrs) as m:
            self.manager_command.execute(synergy_url='', args=mock_parser.args)

        # Check the executed call to "manager start ManagerA"
        m.assert_called_once_with(
            "/synergy/start",
            {"manager": "ManagerA"})

        # Check the data when we call tabulate
        headers = ["manager", "status", "rate (min)"]
        table = [["ManagerA", "RUNNING (started successfully)", 1]]
        mock_tabulate.assert_called_once_with(
            table,
            headers,
            tablefmt="fancy_grid")

    @mock.patch('synergy.client.command.tabulate')
    def test_execute_stop_manager(self, mock_tabulate):
        """Check the CLI output of "manager stop ManagerA"."""
        # Mock the parser call
        mock_parser = mock.Mock()
        mock_parser.args.command = "stop"
        mock_parser.args.manager = "ManagerA"

        # Mock a manager
        manager_a = mock.Mock()
        manager_a.getName.return_value = "ManagerA"
        manager_a.getStatus.return_value = "ACTIVE"
        manager_a.get.return_value = "stopped successfully"
        manager_a.getRate.return_value = 1
        mgrs = manager_a

        # Execute "manager stop ManagerA"
        with mock.patch.object(HTTPCommand, 'execute', return_value=mgrs) as m:
            self.manager_command.execute(synergy_url='', args=mock_parser.args)

        # Check the executed call to "manager stop ManagerA"
        m.assert_called_once_with(
            "/synergy/stop",
            {"manager": "ManagerA"})

        # Check the data when we call tabulate
        headers = ["manager", "status", "rate (min)"]
        table = [["ManagerA", "ACTIVE (stopped successfully)", 1]]
        mock_tabulate.assert_called_once_with(
            table,
            headers,
            tablefmt="fancy_grid")



# coding: utf-8
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Test the HTTPCommand class.

"""

import mock
import requests

from synergy.client.command import HTTPCommand
from synergy.tests import base


class TestHTTPCommand(base.TestCase):

    def setUp(self):
        super(TestHTTPCommand, self).setUp()
        self.http_command = HTTPCommand(name="dummy_httpcmd")

    def test_get_name(self):
        self.assertEqual("dummy_httpcmd", self.http_command.getName())

    def test_configure_parser(self):
        """This method should be implemented in sub-classes."""
        self.assertRaises(
            NotImplementedError,
            self.http_command.configureParser,
            "dummy_subparser")

    def test_execute_success(self):
        mock_response = mock.Mock()
        mock_response.text = '{"test": true}'  # mock a simple json response

        with mock.patch.object(requests, "get", return_value=mock_response)\
                as m:
            result = self.http_command.execute("dummy_url")

        m.assert_called_once_with("dummy_url", headers=None, params=None)
        self.assertEqual({"test": True}, result)



import json
import os.path
import requests

from datetime import datetime
from synergy.client.exception import SynergyError


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""


class Token(object):

    def __init__(self, token, data):
        self.id = token

        data = data["token"]
        self.roles = data["roles"]
        self.catalog = data["catalog"]
        self.issued_at = datetime.strptime(data["issued_at"],
                                           "%Y-%m-%dT%H:%M:%S.%fZ")
        self.expires_at = datetime.strptime(data["expires_at"],
                                            "%Y-%m-%dT%H:%M:%S.%fZ")
        self.project = data["project"]
        self.user = data["user"]

        if "extras" in data:
            self.extras = data["extras"]

    def getCatalog(self):
        return self.catalog

    def getExpiration(self):
        return self.expires_at

    def getId(self):
        return self.id

    def getExtras(self):
        return self.extras

    def getProject(self):
        return self.project

    def getRoles(self):
        return self.roles

    def getUser(self):
        return self.user

    def isAdmin(self):
        if not self.roles:
            return False

        for role in self.roles:
            if role["name"] == "admin":
                return True

        return False

    def issuedAt(self):
        return self.issued_at

    def isExpired(self):
        return self.getExpiration() < datetime.utcnow()

    def save(self, filename):
        # save to file
        with open(filename, 'w') as f:
            token = {}
            token["catalog"] = self.catalog
            token["extras"] = self.extras
            token["user"] = self.user
            token["project"] = self.project
            token["roles"] = self.roles
            token["roles"] = self.roles
            token["issued_at"] = self.issued_at.isoformat()
            token["expires_at"] = self.expires_at.isoformat()

            data = {"id": self.id, "token": token}

            json.dump(data, f)

    @classmethod
    def load(cls, filename):
        if not os.path.isfile(".auth_token"):
            return None

        # load from file:
        with open(filename, 'r') as f:
            try:
                data = json.load(f)
                return Token(data["id"], data)
            # if the file is empty the ValueError will be thrown
            except ValueError as ex:
                raise SynergyError(ex)

    def isotime(self, at=None, subsecond=False):
        """Stringify time in ISO 8601 format."""
        if not at:
            at = datetime.utcnow()

        if not subsecond:
            st = at.strftime('%Y-%m-%dT%H:%M:%S')
        else:
            st = at.strftime('%Y-%m-%dT%H:%M:%S.%f')

        if at.tzinfo:
            tz = at.tzinfo.tzname(None)
        else:
            tz = 'UTC'

        st += ('Z' if tz == 'UTC' else tz)
        return st


class KeystoneClient(object):

    def __init__(self, auth_url, username, password, user_domain_id=None,
                 user_domain_name="default", project_id=None,
                 project_name=None, project_domain_id=None,
                 project_domain_name="default", timeout=None, ca_cert=None):
        self.auth_url = auth_url
        self.username = username
        self.password = password
        self.user_domain_id = user_domain_id
        self.user_domain_name = user_domain_name
        self.project_id = project_id
        self.project_name = project_name
        self.project_domain_id = project_domain_id
        self.project_domain_name = project_domain_name
        self.timeout = timeout
        self.token = None
        self.ca_cert = ca_cert

    def authenticate(self):
        if self.token is not None:
            if self.token.isExpired():
                try:
                    self.deleteToken(self.token.getId())
                except requests.exceptions.HTTPError:
                    pass
            else:
                return

        headers = {"Content-Type": "application/json",
                   "Accept": "application/json",
                   "User-Agent": "python-novaclient"}

        user_domain = {}
        if self.user_domain_id is not None:
            user_domain["id"] = self.user_domain_id
        else:
            user_domain["name"] = self.user_domain_name

        project_domain = {}
        if self.project_domain_id is not None:
            project_domain["id"] = self.project_domain_id
        else:
            project_domain["name"] = self.project_domain_name

        identity = {"methods": ["password"],
                    "password": {"user": {"name": self.username,
                                          "domain": user_domain,
                                          "password": self.password}}}

        data = {"auth": {}}
        data["auth"]["identity"] = identity

        if self.project_name:
            data["auth"]["scope"] = {"project": {"name": self.project_name,
                                                 "domain": project_domain}}

        if self.project_id:
            data["auth"]["scope"] = {"project": {"id": self.project_id,
                                                 "domain": project_domain}}

        response = requests.post(url=self.auth_url + "/auth/tokens",
                                 headers=headers,
                                 data=json.dumps(data),
                                 timeout=self.timeout,
                                 verify=self.ca_cert)

        if response.status_code != requests.codes.ok:
            response.raise_for_status()

        if not response.text:
            raise SynergyError("authentication failed!")

        token_subject = response.headers["X-Subject-Token"]
        token_data = response.json()

        self.token = Token(token_subject, token_data)

        return self.token

    def getToken(self):
        return self.token

    def getService(self, name):
        for service in self.token.getCatalog():
            if service["name"] == name:
                return service

        raise SynergyError("service %s not found!" % name)

    def getEndpoint(self, name, interface="public"):
        service = self.getService(name)

        for endpoint in service["endpoints"]:
            if endpoint["interface"] == interface:
                return endpoint

        raise SynergyError("endpoint for service %s not found!" % name)



import logging

from serializer import SynergyObject
from synergy.exception import SynergyError
from threading import Condition
from threading import Event
from threading import Thread


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""

LOG = logging.getLogger(__name__)


class Manager(SynergyObject, Thread):

    def __init__(self, name=None):
        super(Manager, self).__init__()
        self.condition = Condition()
        self.stop_event = Event()
        self.setDaemon(True)
        self.setName(name)
        self.setStatus("CREATED")
        self.setRate(-1)
        self.config_opts = []
        self.paused = True  # start out paused
        self.managers = {}

    def execute(self, command, *args, **kargs):
        raise NotImplementedError

    def task(self):
        raise NotImplementedError

    def doOnEvent(self, event_type, *args, **kargs):
        raise NotImplementedError

    def getManagers(self):
        return self.managers

    def getManager(self, name):
        return self.managers.get(name, None)

    def notify(self, event_type="DEFAULT", manager_name=None, *args, **kargs):
        if manager_name is not None:
            if manager_name in self.managers:
                self.managers[manager_name].doOnEvent(event_type,
                                                      *args, **kargs)
        else:
            for manager in self.managers.values():
                if manager.getName() != self.getName():
                    try:
                        manager.doOnEvent(event_type, *args, **kargs)
                    except NotImplementedError:
                        pass

    def getOptions(self):
        return self.config_opts

    def getStatus(self):
        return self.get("status")

    def setStatus(self, status):
        with self.condition:
            self.set("status", status)

            self.condition.notifyAll()

    def isAutoStart(self):
        return self.get("autostart")

    def setAutoStart(self, autostart):
        self.set("autostart", autostart)
        self.paused = not autostart

    def getRate(self):
        return self.get("rate")

    def setRate(self, rate):
        self.set("rate", rate)

    def setup(self):
        """Manager initialization

        Hook to do additional manager initialization when one requests
        the service be started. This is called before any service record
        is created.
        Child classes should override this method.
        """
        raise NotImplementedError

    def destroy(self):
        raise NotImplementedError

    def stop(self):
        if self.isAlive():
            # set event to signal thread to terminate
            self.stop_event.set()
            self.resume()
            # block calling thread until thread really has terminated
            self.join()

    def pause(self):
        with self.condition:
            self.paused = True
            self.condition.notifyAll()

    def resume(self):
        with self.condition:
            self.paused = False
            self.condition.notifyAll()

    def run(self):
        while not self.stop_event.isSet():
            with self.condition:
                if self.paused:
                    self.setStatus("ACTIVE")
                    self.condition.wait()
                else:
                    self.setStatus("RUNNING")

                    try:
                        self.task()
                        self.condition.wait(self.getRate() * 60)
                    except NotImplementedError:
                        LOG.debug("task() not implemented by %s"
                                  % self.getName())
                    except SynergyError as ex:
                        LOG.error("task %s: %s" % (self.getName(), ex))



from datetime import datetime
from json import JSONEncoder
from synergy.common import utils
from synergy.exception import SynergyError


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""


class SynergyObject(object):
    """Base class and object factory.

    This forms the base of all objects that can be remoted or instantiated
    via RPC. Simply defining a class that inherits from this base class
    will make it remotely instantiatable. Objects should implement the
    necessary "get" classmethod routines as well as "set" object methods
    as appropriate.
    """

    VERSION = "1.0"

    def __init__(self):
        super(SynergyObject, self).__init__()

        self.attributes = {}

    def getId(self):
        return self.get("id")

    def setId(self, id):
        self.set("id", id)

    def getName(self):
        return self.get("name")

    def setName(self, name):
        self.set("name", name)

    def get(self, field):
        return self.attributes.get(field, None)

    def set(self, field, value):
        if isinstance(value, unicode):
            self.attributes[field] = str(value)
        else:
            self.attributes[field] = value

    def setAttributes(self, attributes):
        if attributes:
            self.attributes = attributes

    @classmethod
    def deserialize(cls, entity):
        if "synergy_object" not in entity:
            raise SynergyError("it seems not a Synergy object!")

        synergy_object = entity["synergy_object"]

        if "namespace" not in synergy_object:
            raise SynergyError("synergy_object.namespace not defined!")

        if "name" not in synergy_object:
            raise SynergyError("synergy_object.name not defined!")

        if "version" not in synergy_object:
            raise SynergyError("synergy_object.version mismatch!")

        if synergy_object["version"] != cls.VERSION:
            raise SynergyError("synergy_object.version mis!")

        if synergy_object["namespace"] != "synergy":
            raise SynergyError("unsupported object objtype='%s.%s"
                               % (synergy_object["namespace"],
                                  synergy_object["name"]))

        objInstance = None

        try:
            objName = synergy_object["name"]
            objClass = utils.import_class(objName)
            objInstance = objClass()
        except Exception as ex:
            raise SynergyError("error on deserializing the object %r: %s"
                               % (objName, ex))

        del entity["synergy_object"]

        entity = utils.objectHookHandler(entity)

        for key, value in entity.items():
            if isinstance(value, dict):
                if "synergy_object" in value:
                    objInstance.set(key, SynergyObject.deserialize(value))
                else:
                    objInstance.set(key, value)
            elif isinstance(value, list):
                l = []

                objInstance.set(key, l)

                for item in value:
                    if isinstance(item, dict) and "synergy_object" in item:
                        l.append(SynergyObject.deserialize(item))
                    else:
                        l.append(item)
            else:
                objInstance.set(key, value)

        return objInstance

    def serialize(self):
        name = self.__class__.__module__ + "." + self.__class__.__name__

        result = {"synergy_object": {}}
        result["synergy_object"]["name"] = name
        result["synergy_object"]["version"] = self.VERSION
        result["synergy_object"]["namespace"] = "synergy"

        for key, value in self.attributes.items():
            if isinstance(value, SynergyObject):
                result[key] = value.serialize()
            elif isinstance(value, dict):
                result[key] = {}

                for k, v in value.items():
                    if isinstance(v, SynergyObject):
                        result[key][k] = v.serialize()
                    else:
                        result[key][k] = v
            elif isinstance(value, list):
                result[key] = []

                for item in value:
                    if isinstance(item, SynergyObject):
                        result[key].append(item.serialize())
                    else:
                        result[key].append(item)
            elif isinstance(value, datetime):
                result[key] = value.isoformat()
            else:
                result[key] = value

        return result


class SynergyEncoder(JSONEncoder):
    def default(self, obj):
        if isinstance(obj, SynergyObject):
            return obj.serialize()
        elif isinstance(obj, datetime):
            return obj.isoformat()
        else:
            return JSONEncoder.default(self, obj)



# coding: utf-8
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Tests a simple implementation of the Manager class.

"""

from synergy.common.manager import Manager
from synergy.tests import base


class SimpleManager(Manager):
    """Simple implementation of the Manager class.

    This class maintains an integer counter.
    The manager task is to increment the counter by one.
    Only one event is possible: increment the counter by 10.
    The counter is set to 0 on setup, not on init.
    """

    def __init__(self, name):
        super(SimpleManager, self).__init__(name)
        self._counter = None

    def _reset_counter(self):
        self._counter = None

    def execute(self, command, *args, **kargs):
        if command == "RESET":
            self._reset_counter()

    def task(self):
        self._counter += 1

    def doOnEvent(self, event_type, *args, **kargs):
        if event_type == "JUMP10":
            self._counter += 10

    def setup(self):
        self._counter = 0

    def destroy(self):
        self._counter = None


class TestManagerImplementation(base.TestCase):

    def setUp(self):
        super(TestManagerImplementation, self).setUp()
        self.manager = SimpleManager(name="simple_manager")
        # We set the manager list to be the manager itself for the sake of
        # simplicity.
        self.manager.managers = {"simple_manager": self.manager}

    def test_execute(self):
        self.manager.setup()
        self.manager.execute("RESET")
        self.assertIsNone(self.manager._counter)

    def test_notify(self):
        self.manager.setup()
        self.manager.notify(event_type="JUMP10", manager_name="simple_manager")
        self.assertEqual(10, self.manager._counter)

    def test_setup(self):
        self.assertIsNone(self.manager._counter)
        self.manager.setup()
        self.assertEqual(0, self.manager._counter)

    def test_destroy(self):
        self.manager.setup()
        self.manager.destroy()
        self.assertIsNone(self.manager._counter)

    def test_do_on_event(self):
        self.manager.setup()
        self.manager.doOnEvent(event_type="JUMP10")
        self.assertEqual(10, self.manager._counter)

    def test_run_stop(self):
        self.manager.setup()
        self.manager.rate = 1.0 / 60 / 1000  # to sleep only 1 ms and not 1 min

        self.manager.start()
        self.assertEqual(True, self.manager.isAlive())

        self.manager.stop()
        self.assertEqual(False, self.manager.isAlive())



import errno
import eventlet
import logging
import os
import re
import socket
import ssl
import time

from eventlet import greenio as eventlet_greenio
from eventlet import wsgi as eventlet_wsgi
from synergy.exception import SynergyError
from sys import exc_info
from traceback import format_tb


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""

LOG = logging.getLogger(__name__)


class Dispatcher(object):
    """Dispatcher

    The main WSGI application. Dispatch the current request to
    the functions from above and store the regular expression
    captures in the WSGI environment as `myapp.url_args` so that
    the functions from above can access the url placeholders.
    If nothing matches call the `not_found` function.
    """

    def __init__(self):
        self.actions = {}

    def register(self, action, callback):
        self.actions[action] = callback

    def unregister(self, action):
        del self.actions[action]

    def __call__(self, environ, start_response):
        """Call the application can catch exceptions."""
        appiter = None
        # just call the application and send the output back unchanged
        # but catch exceptions

        path = environ.get('PATH_INFO', '').lstrip('/')
        application = None

        for regex, callback in self.actions.items():
            match = re.search(regex, path)
            if match is not None:
                environ['myapp.url_args'] = match.groups()
                application = callback
                break

        if application is not None:
            try:
                self.appiter = callback(environ, start_response)
                for item in self.appiter:
                    yield item
            # if an exception occours we get the exception information and
            # prepare a traceback we can render
            except Exception:
                e_type, e_value, tb = exc_info()
                traceback = ['Traceback (most recent call last):']
                traceback += format_tb(tb)
                traceback.append('%s: %s' % (e_type.__name__, e_value))
                # we might have not a stated response by now.
                # Try to start one with the status
                # code 500 or ignore an raised exception if the application
                # already started one.
                try:
                    start_response("500 INTERNAL SERVER ERROR",
                                   [('Content-Type', 'text/plain')])
                except Exception:
                    pass
                yield '\n'.join(traceback)

            # wsgi applications might have a close function.
            # If it exists it *must* be called.
            if hasattr(appiter, 'close'):
                self.appiter.close()
        else:
            """Called if no applations matches."""
            try:
                start_response("404 NOT FOUND",
                               [('Content-Type', 'text/plain')])
            except Exception:
                pass
            yield "Not Found"


class WSGILog(object):
    """A thin wrapper that responds to `write` and logs."""

    def __init__(self, logger, level=20):
        self.logger = logger
        self.level = level

    def write(self, msg):
        self.logger.log(self.level, msg.rstrip())


class Server(object):
    """Server class to manage multiple WSGI sockets and applications."""

    def __init__(self, name, host_name, host_port=8051, threads=1000,
                 application=None, use_ssl=False, ssl_ca_file=None,
                 ssl_cert_file=None, ssl_key_file=None, max_header_line=16384,
                 retry_until_window=30, tcp_keepidle=600, backlog=4096):

        """Parameters

        name: the server's name
        host_name: the host's name
        host_port:
        application:
        backlog: number of backlog requests to configure the socket with
        tcp_keepidle: sets the value of TCP_KEEPIDLE in seconds for each server
        socket. Not supported on OS X
        retry_until_window: number of seconds to keep retrying to listen
        max_header_line: max header line to accommodate large tokens
        use_ssl: enable SSL on the API server
        ssl_ca_file: CA certificate file to use to verify connecting clients
        ssl_cert_file: the certificate file
        ssl_key_file: the private key file
        """

        # Raise the default from 8192 to accommodate large tokens
        eventlet_wsgi.MAX_HEADER_LINE = max_header_line

        self.name = name
        self.host_name = host_name
        self.host_port = host_port
        self.application = application
        self.threads = threads
        self.socket = None
        self.use_ssl = use_ssl
        self.tcp_keepidle = tcp_keepidle
        self.backlog = backlog
        self.retry_until_window = retry_until_window
        self.running = False
        self.dispatcher = Dispatcher()

        if not application:
            self.application = self.dispatcher

        if use_ssl:
            if not os.path.exists(ssl_cert_file):
                raise RuntimeError("Unable to find ssl_cert_file: %s"
                                   % ssl_cert_file)

            if not os.path.exists(ssl_key_file):
                raise RuntimeError("Unable to find ssl_key_file : %s"
                                   % ssl_key_file)

            # ssl_ca_file is optional
            if ssl_ca_file and not os.path.exists(ssl_ca_file):
                raise RuntimeError("Unable to find ssl_ca_file: %s"
                                   % ssl_ca_file)

            self.ssl_kwargs = {
                'server_side': True,
                'certfile': ssl_cert_file,
                'keyfile': ssl_key_file,
                'cert_reqs': ssl.CERT_NONE,
            }

            if ssl_ca_file:
                self.ssl_kwargs['ca_certs'] = ssl_ca_file
                self.ssl_kwargs['cert_reqs'] = ssl.CERT_REQUIRED

    def register(self, action, callback):
        self.dispatcher.register(action, callback)

    def unregister(self, action):
        self.dispatcher.unregister(action)

    def start(self):
        """Run a WSGI server with the given application.

        :param application: The application to be run in the WSGI server
        :param port: Port to bind to if none is specified in conf
        """

        pgid = os.getpid()
        try:
            # NOTE(flaper87): Make sure this process
            # runs in its own process group.
            os.setpgid(pgid, pgid)
        except OSError:
            pgid = 0

        try:
            info = socket.getaddrinfo(self.host_name,
                                      self.host_port,
                                      socket.AF_UNSPEC,
                                      socket.SOCK_STREAM)[0]
            family = info[0]
            bind_addr = info[-1]
        except Exception as ex:
            raise SynergyError("Unable to listen on %s:%s: %s"
                               % (self.host_name, self.host_port, ex))

        retry_until = time.time() + self.retry_until_window
        exception = None

        while not self.socket and time.time() < retry_until:
            try:
                self.socket = eventlet.listen(bind_addr,
                                              backlog=self.backlog,
                                              family=family)
                if self.use_ssl:
                    self.socket = ssl.wrap_socket(self.socket,
                                                  **self.ssl_kwargs)

                if self.use_ssl:
                    ssl.wrap_socket(self.sock, **self.ssl_kwarg)

            except socket.error as ex:
                exception = ex
                LOG.error("Unable to listen on %s:%s: %s"
                          % (self.host_name, self.host_port, ex))

                if ex.errno == errno.EADDRINUSE:
                    retry_until = 0
                    eventlet.sleep(0.1)
                    break

        if exception is not None:
            raise exception

        if not self.socket:
            raise RuntimeError("Could not bind to %s:%s after trying for %d s"
                               % (self.host_name, self.host_port,
                                  self.retry_until_window))

        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

        # sockets can hang around forever without keepalive
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)

        # This option isn't available in the OS X version of eventlet
        if hasattr(socket, 'TCP_KEEPIDLE'):
            self.socket.setsockopt(socket.IPPROTO_TCP,
                                   socket.TCP_KEEPIDLE,
                                   self.tcp_keepidle)

        os.umask(0o27)  # ensure files are created with the correct privileges

        self.pool = eventlet.GreenPool(self.threads)
        self.pool.spawn_n(self._single_run, self.application, self.socket)

        self.running = True

    def isRunning(self):
        return self.running

    def stop(self):
        LOG.info("shutting down: requests left: %s", self.pool.running())
        self.running = False
        self.pool.resize(0)
        # self.pool.waitall()

        if self.socket:
            eventlet_greenio.shutdown_safe(self.socket)
            self.socket.close()

        self.running = False

    def wait(self):
        """Wait until all servers have completed running"""
        try:
            self.pool.waitall()
        except KeyboardInterrupt:
            pass

    def _single_run(self, application, sock):
        """Start a WSGI server in a new green thread."""
        LOG.info("Starting single process server")
        eventlet_wsgi.server(sock, application,
                             custom_pool=self.pool,
                             log=WSGILog(LOG),
                             debug=False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


try:
    import unittest2 as unittest
except ImportError:
    import unittest

import json
import logging
import mock
import sys
import time

from mock import Mock
from synergy.common.utils import objectHookHandler
from synergy.service import Synergy

logging.basicConfig(level=logging.DEBUG)

LOG = logging.getLogger("SynergyTests")
LOG.setLevel(logging.DEBUG)


def getLogger(name):
    formatter = logging.Formatter("%(asctime)s - %(name)s - "
                                  "%(levelname)s - %(message)s")

    ch = logging.StreamHandler(sys.stdout)
    ch.setLevel(logging.DEBUG)
    ch.setFormatter(formatter)

    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    logger.propagate = False
    logger.addHandler(ch)

    return logger


class SynergyTests(unittest.TestCase):

    @mock.patch('synergy.service.LOG', LOG)
    def setUp(self):
        super(SynergyTests, self).setUp()

        self.synergy = Synergy()
        self.synergy.managers["TimerManager"].start()
        time.sleep(1)

    @mock.patch('synergy.service.LOG', LOG)
    def test_managers(self):
        self.assertEqual(self.synergy.managers.keys(), ["TimerManager"])

    @mock.patch('synergy.service.LOG', LOG)
    def test_listManagers(self):
        start_response = Mock()
        result = self.synergy.listManagers(environ={},
                                           start_response=start_response)

        result = json.loads(result[0], object_hook=objectHookHandler)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0].getName(), "TimerManager")

    @mock.patch('synergy.service.LOG', LOG)
    def test_getManagerStatus(self):
        start_response = Mock()
        environ = {}
        result = self.synergy.getManagerStatus(environ,
                                               start_response=start_response)

        result = json.loads(result[0], object_hook=objectHookHandler)

        self.assertEqual(result[0].getStatus(), 'ACTIVE')

        environ = {'QUERY_STRING': 'manager=TimerManager'}

        result = self.synergy.getManagerStatus(environ,
                                               start_response=start_response)

        result = json.loads(result[0], object_hook=objectHookHandler)

        self.assertEqual(result[0].getStatus(), 'ACTIVE')

    @mock.patch('synergy.service.LOG', LOG)
    def test_startManager(self):
        start_response = Mock()
        environ = {'QUERY_STRING': 'manager=NONE'}

        result = self.synergy.startManager(environ, start_response)

        self.assertEqual(result[0], "manager NONE not found!")

        environ = {'QUERY_STRING': 'manager=TimerManager'}

        result = self.synergy.startManager(environ, start_response)
        result = json.loads(result[0], object_hook=objectHookHandler)

        self.assertEqual(result.getStatus(), 'RUNNING')
        self.assertEqual(result.get("message"), 'started successfully')

        time.sleep(0.5)

        result = self.synergy.startManager(environ, start_response)
        result = json.loads(result[0], object_hook=objectHookHandler)

        self.assertEqual(result.getStatus(), 'RUNNING')
        self.assertEqual(result.get("message"), 'WARN: already started')

    @mock.patch('synergy.service.LOG', LOG)
    def test_stopManager(self):
        stop_response = Mock()
        environ = {'QUERY_STRING': 'manager=NONE'}

        result = self.synergy.startManager(environ, stop_response)

        self.assertEqual(result[0], "manager NONE not found!")

        environ = {'QUERY_STRING': 'manager=TimerManager'}

        result = self.synergy.startManager(environ, stop_response)
        result = json.loads(result[0], object_hook=objectHookHandler)

        time.sleep(0.5)

        result = self.synergy.stopManager(environ, stop_response)
        result = json.loads(result[0], object_hook=objectHookHandler)

        self.assertEqual(result.getStatus(), 'ACTIVE')

    @mock.patch('synergy.service.LOG', LOG)
    def test_executeCommand(self):
        environ = {'QUERY_STRING':
                   'manager=TimerManager&args=%7B%22id%22%3A'
                   '+null%2C+%22name%22%3A+%22prj_a%22%7D&command=GET_TIME'}
        start_response = Mock()

        result = self.synergy.executeCommand(environ, start_response)
        result = json.loads(result[0])

        self.assertEqual(result.keys(), ["localtime"])


if __name__ == '__main__':
    unittest.main()



import logging
import time

from synergy.common.manager import Manager
from synergy.exception import SynergyError


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""

LOG = logging.getLogger(__name__)


class TimerManager(Manager):

    def __init__(self):
        Manager.__init__(self, name="TimerManager")

    def setup(self):
        LOG.info("%s setup invoked!" % (self.name))

    def execute(self, command, *args, **kargs):
        LOG.info("%r execute %r invoked!" % (self.name, command))

        if command == "GET_TIME":
            return {"localtime": time.asctime(time.localtime(time.time()))}
        else:
            raise SynergyError("command %r not supported" % command)

    def destroy(self):
        LOG.info("%s destroy invoked!" % (self.name))

    def task(self):
        localtime = time.asctime(time.localtime(time.time()))
        LOG.info("Local current time: %s" % (localtime))



from oslo_config import cfg

__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""


CONF = cfg.CONF

auth_opts = [
    cfg.StrOpt("plugin", default="noauth", help="the authorization plugin"),
    cfg.StrOpt("policy_file", default="policy.json", help="the plucy file"),
]

wsgi_opts = [
    cfg.StrOpt("host", default="localhost",
               help="Address to bind the server"),
    cfg.IntOpt("port", default=8051,
               help="The port on which the server will listen"),
    cfg.IntOpt("threads", default=1000),
    cfg.IntOpt("backlog", default=4096,
               help="Number of backlog requests to configure the socket with"),
    cfg.IntOpt("tcp_keepidle", default=600,
               help="Sets the value of TCP_KEEPIDLE in seconds for each server"
                    " socket (not supported on OS X)"),
    cfg.IntOpt("retry_until_window", default=30,
               help="Number of seconds to keep retrying to listen"),
    cfg.IntOpt("max_header_line", default=16384,
               help="Max header line to accommodate large tokens"),
    cfg.BoolOpt("use_ssl", default=False,
                help="Enable SSL on the API server"),
    cfg.StrOpt("ssl_ca_file", default=None,
               help="CA certificate file to use to verify connecting clients"),
    cfg.StrOpt("ssl_cert_file", default=None,
               help="The certificate file"),
    cfg.StrOpt("ssl_key_file", default=None,
               help="The private key file")
]

logger_opts = [
    cfg.StrOpt("filename", default="/tmp/synergy.log",
               required=True),
    cfg.StrOpt("level", default="INFO", required=False),
    cfg.IntOpt("maxBytes", default=1048576),
    cfg.IntOpt("backupCount", default=100),
    cfg.StrOpt("formatter",
               default="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
               required=False)
]

manager_opts = [
    cfg.BoolOpt("autostart", default=False),
    cfg.IntOpt("rate", default=60)
]

cfg.CONF.register_opts(wsgi_opts, group="WSGI")
cfg.CONF.register_opts(logger_opts, group="Logger")
cfg.CONF.register_opts(auth_opts, group="Authorization")


def parseArgs(args=None, usage=None, default_config_files=None):
    cfg.CONF(args=args,
             project='synergy',
             version="1.0",
             usage=usage,
             default_config_files=default_config_files)



import eventlet
import json
import logging
import logging.handlers
import os
import sys

from cgi import escape
from cgi import parse_qs
from pkg_resources import iter_entry_points

from oslo_config import cfg

from synergy.common import config
from synergy.common.manager import Manager
from synergy.common.serializer import SynergyEncoder
from synergy.common.service import Service
from synergy.common import utils
from synergy.common.wsgi import Server
from synergy.exception import AuthorizationError
from synergy.exception import SynergyError


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""

CONF = cfg.CONF
LOG = None
MANAGER_ENTRY_POINT = "synergy.managers"  # used to discover Synergy managers


def setLogger(name):
    """Configure the given logger with Synergy logging configuration.

    Note:
    This function should only be used when entering Synergy by the main()
    function. Otherwise you may run into issues due to logging to protected
    files.
    """
    # create a logging format
    formatter = logging.Formatter(CONF.Logger.formatter)

    log_dir = os.path.dirname(CONF.Logger.filename)
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)

    # Add the log message handler to the logger
    handler = logging.handlers.RotatingFileHandler(
        CONF.Logger.filename,
        maxBytes=CONF.Logger.maxBytes,
        backupCount=CONF.Logger.backupCount)

    handler.setFormatter(formatter)

    # set logger level
    logger = logging.getLogger(name)

    if not len(logger.handlers):
        try:
            logger.setLevel(cfg.CONF.Logger.level)
        except ValueError:  # wrong level, we default to INFO
            logger.setLevel(logging.INFO)

        logger.propagate = False
        logger.addHandler(handler)


class Synergy(Service):
    """Service object for binaries running on hosts.

    A service takes a manager and enables rpc by listening to queues based
    on topic. It also periodically runs tasks on the manager and reports
    it state to the database services table.
    """

    def __init__(self, *args, **kwargs):
        super(Synergy, self).__init__("Synergy")

        self.managers = {}
        self.wsgi_server = None
        self.auth_plugin = CONF.Authorization.plugin

        if self.auth_plugin == "noauth":
            LOG.info("the authorization is disabled!")
            self.auth_plugin = None
        else:
            LOG.info("loading the auth_plugin %s" % self.auth_plugin)
            self.auth_plugin = utils.instantiate_class(self.auth_plugin)

        for entry in iter_entry_points(MANAGER_ENTRY_POINT):
            LOG.info("loading the %s manager", entry.name)

            try:
                CONF.register_opts(config.manager_opts, group=entry.name)

                manager_class = entry.load()

                manager_obj = manager_class(*args, **kwargs)
                manager_obj.setName(entry.name)
                manager_obj.setAutoStart(CONF.get(entry.name).autostart)
                manager_obj.setRate(CONF.get(entry.name).rate)

                # Configure logging for manager
                setLogger(manager_obj.__module__.rsplit('.', 1)[0])
                setLogger(manager_obj.__module__)

                self.managers[manager_obj.getName()] = manager_obj

                CONF.register_opts(manager_obj.getOptions(), group=entry.name)
            except cfg.Error as ex:
                LOG.error("Exception has occured", exc_info=1)

                LOG.error("manager %s instantiation error: %s"
                          % (entry.name, ex))

                raise SynergyError("manager %s instantiation error: %s"
                                   % (entry.name, ex))

        for name, manager in self.managers.items():
            manager.managers = self.managers

            try:
                LOG.info("initializing the %s manager" % (manager.getName()))

                manager.setup()

                LOG.info("manager %s initialized!" % (manager.getName()))
            except NotImplementedError:
                message = "manager %s instantiation error: setup() not " \
                          "implemented!" % name

                LOG.error(message)
                raise SynergyError(message)
            except SynergyError as ex:
                LOG.error("manager %s instantiation error: %s" % (name, ex))
                self.managers[manager.getName()].setStatus("ERROR")
                raise ex

        self.saved_args, self.saved_kwargs = args, kwargs

    def parseParameters(f):
        def wrapper(self, *args, **kw):
            if not args:
                return f(self, *args, **kw)

            context = args[0]

            query = context.get("QUERY_STRING", None)

            if query:
                parameters = parse_qs(query)

                for key in parameters:
                    value = escape(parameters[key][0])
                    value = value.replace("'", "\"")

                    try:
                        value = json.loads(value)
                    except ValueError:
                        pass

                    context[key] = value

            return f(self, *args, **kw)

        return wrapper

    def checkParameters(parameters):
        def check(f):
            def wrapper(self, *args, **kw):
                context = args[0]
                start_response = args[1]

                for parameter in parameters:
                    value = context.get(parameter, None)

                    if not value:
                        start_response("400 BAD REQUEST",
                                       [("Content-Type", "text/plain")])
                        return ["parameter %s not found!" % parameter]

                    if parameter == "manager" and value not in self.managers:
                        start_response("404 NOT FOUND",
                                       [("Content-Type", "text/plain")])
                        return ["manager %s not found!" % value]

                return f(self, *args, **kw)
            return wrapper
        return check

    def authorize(f):
        def wrapper(self, *args, **kw):
            if self.auth_plugin:
                context = args[0]
                context["managers"] = self.managers

                try:
                    self.auth_plugin.authorize(context)
                except AuthorizationError as ex:
                    args[1]("401 Unauthorized",
                            [("Content-Type", "text/plain")])
                    return ["%s" % ex.message]

            return f(self, *args, **kw)

        return wrapper

    @authorize
    def listManagers(self, environ, start_response):
        result = []

        for name, manager in self.managers.items():
            m = Manager(name)
            m.setStatus(manager.getStatus())
            m.setRate(manager.getRate())

            result.append(m)

        start_response("200 OK", [("Content-Type", "text/html")])
        return ["%s" % json.dumps(result, cls=SynergyEncoder)]

    @parseParameters
    @authorize
    def getManagerStatus(self, environ, start_response):
        manager_name = environ.get("manager", None)

        manager_list = None
        result = []

        if manager_name:
            if manager_name not in self.managers:
                start_response("404 NOT FOUND",
                               [("Content-Type", "text/plain")])
                return "manager %s not found!" % manager_name

            manager_list = [manager_name]
        else:
            manager_list = self.managers.keys()

        for manager_name in manager_list:
            manager_name = escape(manager_name)

            if manager_name in self.managers:
                manager = self.managers[manager_name]

                m = Manager(manager_name)
                m.setStatus(manager.getStatus())
                m.setRate(manager.getRate())

                result.append(m)

        start_response("200 OK", [("Content-Type", "text/html")])
        return [json.dumps(result, cls=SynergyEncoder)]

    @parseParameters
    @checkParameters(["manager", "command"])
    @authorize
    def executeCommand(self, environ, start_response):
        manager_name = environ["manager"]
        manager = self.managers[manager_name]
        manager_args = environ["args"]
        command = environ["command"]

        try:
            result = manager.execute(command=command, **manager_args)

            start_response("200 OK", [("Content-Type", "text/html")])
            return [json.dumps(result, cls=SynergyEncoder)]
        except NotImplementedError:
            message = "execute() not implemented!"

            LOG.error(message)
            start_response("500 INTERNAL SERVER ERROR",
                           [("Content-Type", "text/plain")])
            return [message]

        except SynergyError as ex:
            LOG.debug("execute command: error=%s" % ex)

            start_response("422 Unprocessable Entity",
                           [("Content-Type", "text/plain")])
            return ["%s" % ex.message]

    @parseParameters
    @checkParameters(["manager"])
    @authorize
    def startManager(self, environ, start_response):
        manager_name = environ["manager"]
        manager = self.managers[manager_name]
        result = Manager(manager_name)
        result.setRate(manager.getRate())

        if manager.getStatus() == "ACTIVE":
            LOG.info("starting the %s manager" % (manager_name))

            manager.resume()

            LOG.info("%s manager started! (rate=%s min)"
                     % (manager_name, manager.getRate()))

            result.setStatus("RUNNING")
            result.set("message", "started successfully")
        elif manager.getStatus() == "RUNNING":
            result.setStatus("RUNNING")
            result.set("message", "WARN: already started")
        elif manager.getStatus() == "ERROR":
            result.setStatus("ERROR")
            result.set("message", "wrong state")

        start_response("200 OK", [("Content-Type", "text/html")])
        return [json.dumps(result, cls=SynergyEncoder)]

    @parseParameters
    @checkParameters(["manager"])
    @authorize
    def stopManager(self, environ, start_response):
        manager_name = environ["manager"]
        manager = self.managers[manager_name]
        result = Manager(manager_name)
        result.setRate(manager.getRate())

        if manager.getStatus() == "RUNNING":
            LOG.info("stopping the %s manager" % (manager_name))

            manager.pause()

            LOG.info("%s manager stopped!" % (manager_name))

            result.setStatus("ACTIVE")
            result.set("message", "stopped successfully")
        elif manager.getStatus() == "ACTIVE":
            result.setStatus("ACTIVE")
            result.set("message", "WARN: already stopped")
        elif manager.getStatus() == "ERROR":
            result.setStatus("ERROR")
            result.set("message", "wrong state")

        start_response("200 OK", [("Content-Type", "text/html")])
        return [json.dumps(result, cls=SynergyEncoder)]

    def start(self):
        self.model_disconnected = False

        for name, manager in self.managers.items():
            if manager.getStatus() != "ERROR":
                try:
                    LOG.info("starting the %s manager" % (name))
                    manager.start()

                    LOG.info("%s manager started! (rate=%s min, status=%s)"
                             % (name, manager.getRate(), manager.getStatus()))
                except SynergyError as ex:
                    LOG.error("error occurred during the manager start %s"
                              % (ex))
                    manager.setStatus("ERROR")
                    raise ex

        self.wsgi_server = Server(
            name="WSGI server",
            host_name=CONF.WSGI.host,
            host_port=CONF.WSGI.port,
            threads=CONF.WSGI.threads,
            use_ssl=CONF.WSGI.use_ssl,
            ssl_ca_file=CONF.WSGI.ssl_ca_file,
            ssl_cert_file=CONF.WSGI.ssl_cert_file,
            ssl_key_file=CONF.WSGI.ssl_key_file,
            max_header_line=CONF.WSGI.max_header_line,
            retry_until_window=CONF.WSGI.retry_until_window,
            tcp_keepidle=CONF.WSGI.tcp_keepidle,
            backlog=CONF.WSGI.backlog)

        self.wsgi_server.register(r'synergy/list', self.listManagers)
        self.wsgi_server.register(r'synergy/status', self.getManagerStatus)
        self.wsgi_server.register(r'synergy/execute', self.executeCommand)
        self.wsgi_server.register(r'synergy/start', self.startManager)
        self.wsgi_server.register(r'synergy/stop', self.stopManager)
        self.wsgi_server.start()

        LOG.info("STARTED!")
        self.wsgi_server.wait()

    def kill(self):
        """Destroy the service object in the datastore."""
        LOG.warn("killing service")
        self.stop()
        LOG.warn("Service killed")

    def stop(self):
        for name, manager in self.managers.items():
            LOG.info("destroying the %s manager" % (name))
            try:
                manager.setStatus("DESTROYED")
                manager.destroy()
                # manager.join()
                # LOG.info("%s manager destroyed" % (name))
            except NotImplementedError:
                message = "method destroy() not implemented by the " \
                          "%s manager" % manager.getName()

                LOG.error(message)

            except SynergyError as ex:
                LOG.error("Exception has occured", exc_info=1)

                manager.setStatus("ERROR")
                LOG.error("error occurred during the manager destruction: %s"
                          % ex)

        if self.wsgi_server:
            self.wsgi_server.stop()

        LOG.info("STOPPED!")


def main():
    try:
        eventlet.monkey_patch(os=False)

        # the configuration will be into the cfg.CONF global data structure
        config.parseArgs(args=sys.argv[1:],
                         default_config_files=["/etc/synergy/synergy.conf"])

        if not cfg.CONF.config_file:
            sys.exit("ERROR: Unable to find configuration file via the "
                     "default search paths (~/.synergy/, ~/, /etc/synergy/"
                     ", /etc/) and the '--config-file' option!")

        setLogger(name="synergy")
        setLogger(name="oslo.messaging._drivers")

        global LOG

        LOG = logging.getLogger(__name__)
        LOG.info("Starting Synergy...")

        server = Synergy()
        server.start()

        LOG.info("Synergy started")
    except Exception as ex:
        LOG.error("unrecoverable error: %s" % ex)



# coding: utf-8
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Test the ExecuteCommand class.

"""

import mock

from synergy.client.command import ExecuteCommand
from synergy.client.command import HTTPCommand
from synergy.tests import base


class TestExecuteCommand(base.TestCase):

    def setUp(self):
        super(TestExecuteCommand, self).setUp()
        self.execute_command = ExecuteCommand(name="dummy_execmd")

    def test_execute_no_args(self):
        """Check the URL and payload."""
        with mock.patch.object(HTTPCommand, 'execute') as m:
            self.execute_command.execute(
                synergy_url="",
                manager="Manager",
                command="command")

        expected_payload = {
            "manager": "Manager",
            "command": "command",
            "args": "{}"}
        m.assert_called_once_with("/synergy/execute", expected_payload)

    def test_execute_with_args(self):
        """Check the URL and payload."""
        with mock.patch.object(HTTPCommand, 'execute') as m:
            self.execute_command.execute(
                synergy_url="",
                manager="Manager",
                command="command",
                args={"test": {"what": "yes"}})

        expected_payload = {
            "manager": "Manager",
            "command": "command",
            "args": '{"test": {"what": "yes"}}'}
        m.assert_called_once_with("/synergy/execute", expected_payload)



import json
import requests

from synergy.common.utils import objectHookHandler
from tabulate import tabulate


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""


class HTTPCommand(object):

    def __init__(self, name):
        self.name = name
        self.token = None

    def getName(self):
        return self.name

    def setToken(self, token):
        self.token = token

    def configureParser(self, subparser):
        raise NotImplementedError("not implemented!")

    def execute(self, synergy_url, payload=None):
        headers = None

        if self.token:
            headers = {"Content-Type": "application/json",
                       "Accept": "application/json",
                       "User-Agent": "synergy_client",
                       "X-Auth-Token": self.token.getId()}

        request = requests.get(synergy_url, headers=headers, params=payload)
        request.raise_for_status()

        try:
            self.results = json.loads(
                request.text,
                object_hook=objectHookHandler)
        except ValueError:
            self.results = request.json()

        return self.results

    def getResults(self):
        return self.results


class ManagerCommand(HTTPCommand):

    def __init__(self):
        super(ManagerCommand, self).__init__("Manager")

    def configureParser(self, subparser):
        manager_parser = subparser.add_parser('manager')
        manager_parsers = manager_parser.add_subparsers(dest="command")

        manager_parsers.add_parser(
            "list", add_help=True, help="list the managers")

        status_parser = manager_parsers.add_parser(
            "status", add_help=True, help="show the managers status")

        status_parser.add_argument(
            "manager", nargs='?', default=None, help="at most one manager")

        start_parser = manager_parsers.add_parser(
            "start", add_help=True, help="start the manager")

        start_parser.add_argument("manager", help="the manager to start")

        stop_parser = manager_parsers.add_parser(
            "stop", add_help=True, help="stop the manager")

        stop_parser.add_argument(
            "manager", help="the manager to stop")

    def execute(self, synergy_url, args=None):
        table = []
        headers = []
        url = synergy_url

        if args.command == "list":
            headers.append("manager")
            url += "/synergy/list"

            managers = super(ManagerCommand, self).execute(url)

            for manager in managers:
                table.append([manager.getName()])
        else:
            headers.append("manager")
            headers.append("status")
            headers.append("rate (min)")
            url += "/synergy/" + args.command

            result = super(ManagerCommand, self).execute(
                url, {"manager": args.manager})

            if args.command == "status":
                for manager in result:
                    table.append([manager.getName(),
                                  manager.getStatus(),
                                  manager.getRate()])
            else:
                msg = result.get("message")
                table.append([result.getName(),
                              result.getStatus() + " (%s)" % msg,
                              result.getRate()])

        print(tabulate(table, headers, tablefmt="fancy_grid"))


class ExecuteCommand(HTTPCommand):

    def __init__(self, name):
        super(ExecuteCommand, self).__init__(name)

    def execute(self, synergy_url, manager, command, args=None):
        if args is None:
            args = {}

        url = synergy_url + "/synergy/execute"

        payload = {"manager": manager,
                   "command": command,
                   "args": json.dumps(args)}

        return super(ExecuteCommand, self).execute(url, payload)



from synergy.exception import AuthorizationError


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

       http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
       either express or implied.
       See the License for the specific language governing
       permissions and limitations under the License."""


class LocalHostAuthorization(object):

    def authorize(self, context):
        server_addr = context.get("SERVER_NAME")
        remote_addr = context.get("REMOTE_ADDR")

        if not server_addr or not remote_addr or server_addr != remote_addr:
            raise AuthorizationError("You are not authorized!")



from synergy.common import serializer

__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""


class Command(serializer.SynergyObject):
    VERSION = "1.0"

    def __init__(self, name):
        super(Command, self).__init__(name)

    def getParameters(self):
        parameters = self.get("parameters")

        if parameters is None:
            self.set("parameters", {})

        return self.get("parameters")

    def addParameter(self, name, value):
        self.getParameters()[name] = value

    def getParameter(self, name):
        return self.getParameters().get(name, None)

    def setParameters(self, parameters):
        self.set("parameters", parameters)

    def getResults(self):
        result = self.get("result")

        if not result:
            self.set("result", {})

        return self.get("result")

    def addResult(self, name, value):
        self.getResults()[name] = value

    def getResult(self, name):
        return self.getResults().get(name, None)

    def setResults(self, data):
        self.set("result", data)



import logging
import os
import signal
import sys

from oslo_config import cfg


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""

CONF = cfg.CONF
LOG = logging.getLogger(__name__)
SIGTERM_SENT = False


class Service(object):
    def __init__(self, name):
        self.name = name
        signal.signal(signal.SIGTERM, self.sigterm_handler)
        signal.signal(signal.SIGINT, self.sigterm_handler)

    def sigterm_handler(self, signum, frame):
        LOG.debug("Signal handler called with signal=%s" % signum)

        global SIGTERM_SENT
        if not SIGTERM_SENT:
            LOG.info("Shutting down %s" % self.name)
            SIGTERM_SENT = True
            self.stop()
            os.killpg(0, signal.SIGTERM)

        sys.exit()

    def getName(self):
        return self.name

    def start(self):
        raise NotImplementedError

    def stop(self):
        raise NotImplementedError

    def wait(self):
        raise NotImplementedError

    def restart(self):
        # Reload config files and restart service
        CONF.reload_config_files()
        self.stop()
        self.start()



# coding: utf-8
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Test the shell.

"""

import mock

from argparse import Namespace
from synergy.client import keystone_v3
from synergy.client.shell import main
from synergy.tests import base


class TestHTTPCommand(base.TestCase):

    def setUp(self):
        super(TestHTTPCommand, self).setUp()

    @mock.patch('synergy.client.command.ManagerCommand.execute')
    @mock.patch('sys.argv')
    def test_create_keystone_client(self, mock_argv, _mock_httpcmd):
        """Check success when all parameters are filled."""
        args = [
            '--os-username', 'username',
            '--os-password', 'password',
            '--os-user-domain-id', 'user_domain_id',
            '--os-user-domain-name', 'user_domain_name',
            '--os-project-name', 'project_name',
            '--os-project-domain-id', 'project_domain_id',
            '--os-project-domain-name', 'project_domain_name',
            '--os-auth-url', 'auth_url',
            '--os-cacert', 'cacert',
            'manager', 'list']
        mock_argv.__getitem__.return_value = args

        with mock.patch.object(keystone_v3, 'KeystoneClient') as m:
            main()
        m.assert_called_once_with(
            auth_url='auth_url',
            username='username',
            password='password',
            ca_cert='cacert',
            user_domain_id='user_domain_id',
            user_domain_name='user_domain_name',
            project_name='project_name',
            project_domain_id='project_domain_id',
            project_domain_name='project_domain_name')

    @mock.patch('synergy.client.command.ManagerCommand.execute')
    @mock.patch('sys.argv')
    def test_no_username(self, mock_argv, _mock_httpcmd):
        """CLI should exit if no username is specified."""
        args = [
            '--os-password', 'password',
            '--os-user-domain-id', 'user_domain_id',
            '--os-user-domain-name', 'user_domain_name',
            '--os-project-name', 'project_name',
            '--os-project-domain-id', 'project_domain_id',
            '--os-project-domain-name', 'project_domain_name',
            '--os-auth-url', 'auth_url',
            '--os-cacert', 'cacert',
            'manager', 'list']
        mock_argv.__getitem__.return_value = args

        with mock.patch('sys.exit') as m:
            main()
        m.assert_called_once_with(1)

    @mock.patch('synergy.client.command.ManagerCommand.execute')
    @mock.patch('sys.argv')
    def test_no_password(self, mock_argv, _mock_httpcmd):
        """CLI should exit if no password is specified."""
        args = [
            '--os-username', 'username',
            '--os-user-domain-id', 'user_domain_id',
            '--os-user-domain-name', 'user_domain_name',
            '--os-project-name', 'project_name',
            '--os-project-domain-id', 'project_domain_id',
            '--os-project-domain-name', 'project_domain_name',
            '--os-auth-url', 'auth_url',
            '--os-cacert', 'cacert',
            'manager', 'list']
        mock_argv.__getitem__.return_value = args

        with mock.patch('sys.exit') as m:
            main()
        m.assert_called_once_with(1)

    @mock.patch('synergy.client.command.ManagerCommand.execute')
    @mock.patch('sys.argv')
    def test_no_project_name(self, mock_argv, _mock_httpcmd):
        """CLI should exit if no project name is specified."""
        args = [
            '--os-username', 'username',
            '--os-password', 'password',
            '--os-user-domain-id', 'user_domain_id',
            '--os-user-domain-name', 'user_domain_name',
            '--os-project-domain-id', 'project_domain_id',
            '--os-project-domain-name', 'project_domain_name',
            '--os-auth-url', 'auth_url',
            '--os-cacert', 'cacert',
            'manager', 'list']
        mock_argv.__getitem__.return_value = args

        with mock.patch('sys.exit') as m:
            main()
        m.assert_called_once_with(1)

    @mock.patch('synergy.client.command.ManagerCommand.execute')
    @mock.patch('sys.argv')
    def test_no_auth_url(self, mock_argv, _mock_httpcmd):
        """CLI should exit if no auth URL is specified."""
        args = [
            '--os-username', 'username',
            '--os-password', 'password',
            '--os-user-domain-id', 'user_domain_id',
            '--os-user-domain-name', 'user_domain_name',
            '--os-project-name', 'project_name',
            '--os-project-domain-id', 'project_domain_id',
            '--os-project-domain-name', 'project_domain_name',
            '--os-cacert', 'cacert',
            'manager', 'list']
        mock_argv.__getitem__.return_value = args

        with mock.patch('sys.exit') as m:
            main()
        m.assert_called_once_with(1)

    @mock.patch('synergy.client.command.ManagerCommand.execute')
    @mock.patch('sys.argv')
    def test_minimum_parameters(self, mock_argv, _mock_httpcmd):
        """CLI should not exit when the required parameters are filled."""
        args = [
            '--os-username', 'username',
            '--os-password', 'password',
            '--os-project-name', 'project_name',
            '--os-auth-url', 'auth_url',
            'manager', 'list']
        mock_argv.__getitem__.return_value = args

        # Bypass HTTP call
        with mock.patch.object(keystone_v3, 'KeystoneClient'):
            with mock.patch('sys.exit') as sys_exit:
                main()
        sys_exit.assert_not_called()

    @mock.patch('synergy.client.command.ManagerCommand.execute')
    @mock.patch('sys.argv')
    def test_bypass_url(self, mock_argv, mock_httpcmd):
        """Filling bypass-url should set other params as defaults."""
        args = [
            '--os-username', 'username',
            '--os-password', 'password',
            '--os-user-domain-id', 'user_domain_id',
            '--os-user-domain-name', 'user_domain_name',
            '--os-project-name', 'project_name',
            '--os-project-domain-id', 'project_domain_id',
            '--os-project-domain-name', 'project_domain_name',
            '--os-cacert', 'cacert',
            '--os-auth-url', 'auth_url',
            '--bypass-url', 'bypass_url',
            'manager', 'list']
        mock_argv.__getitem__.return_value = args

        with mock.patch.object(keystone_v3, 'KeystoneClient'):
            main()

        ns = Namespace(
            bypass_url='bypass_url',
            command='list',
            command_name='manager',
            debug=False,
            os_auth_url='auth_url',
            os_cacert='cacert',
            os_password='password',
            os_project_domain_id='project_domain_id',
            os_project_domain_name='project_domain_name',
            os_project_id=None,
            os_project_name='project_name',
            os_user_domain_id='user_domain_id',
            os_user_domain_name='user_domain_name',
            os_username='username')
        mock_httpcmd.assert_called_once_with("bypass_url", ns)



import sys
import traceback

from datetime import datetime
from synergy.exception import SynergyError


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""


def import_class(import_str):
    """Returns a class from a string including module and class."""
    mod_str, _sep, class_str = import_str.rpartition('.')
    __import__(mod_str)

    try:
        return getattr(sys.modules[mod_str], class_str)
    except AttributeError:
        raise SynergyError(
            'Class %s cannot be found (%s)' %
            (class_str, traceback.format_exception(*sys.exc_info())))


def instantiate_class(class_str):
    return import_class(class_str)()


def objectHookHandler(json_dict):
    for key, value in json_dict.items():
        if isinstance(value, dict):
            json_dict[key] = objectHookHandler(value)
        else:
            try:
                json_dict[key] = datetime.strptime(value,
                                                   "%Y-%m-%dT%H:%M:%S.%f")
            except Exception as ex:
                pass

    if "synergy_object" in json_dict:
        synergy_object = json_dict["synergy_object"]
        try:
            objClass = import_class(synergy_object["name"])
            objInstance = objClass()
            return objInstance.deserialize(json_dict)
        except SynergyError as ex:
            raise ex
    else:
        return json_dict



import os
import os.path
import sys

from argparse import ArgumentParser
from pkg_resources import iter_entry_points
from requests.exceptions import ConnectionError
from requests.exceptions import HTTPError
from requests.exceptions import RequestException
from synergy.client import keystone_v3

__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied.
See the License for the specific language governing
permissions and limitations under the License."""

COMMANDS_ENTRY_POINT = "synergy.commands"  # used to discover Synergy commands


def main():
    try:
        parser = ArgumentParser(prog="synergy",
                                epilog="Command-line interface to the"
                                       " OpenStack Synergy API.")

        # Global arguments
        parser.add_argument("--version", action="version", version="v1.1")

        parser.add_argument("--debug",
                            default=False,
                            action="store_true",
                            help="print debugging output")

        parser.add_argument("--os-username",
                            metavar="<auth-user-name>",
                            default=os.environ.get("OS_USERNAME"),
                            help="defaults to env[OS_USERNAME]")

        parser.add_argument("--os-password",
                            metavar="<auth-password>",
                            default=os.environ.get("OS_PASSWORD"),
                            help="defaults to env[OS_PASSWORD]")

        parser.add_argument("--os-user-domain-id",
                            metavar="<auth-user-domain-id>",
                            default=os.environ.get("OS_USER_DOMAIN_ID"),
                            help="defaults to env[OS_USER_DOMAIN_ID]")

        parser.add_argument("--os-user-domain-name",
                            metavar="<auth-user-domain-name>",
                            default=os.environ.get("OS_USER_DOMAIN_NAME"),
                            help="defaults to env[OS_USER_DOMAIN_NAME]")

        parser.add_argument("--os-project-name",
                            metavar="<auth-project-name>",
                            default=os.environ.get("OS_PROJECT_NAME"),
                            help="defaults to env[OS_PROJECT_NAME]")

        parser.add_argument("--os-project-id",
                            metavar="<auth-project-id>",
                            default=os.environ.get("OS_PROJECT_ID"),
                            help="defaults to env[OS_PROJECT_ID]")

        parser.add_argument("--os-project-domain-id",
                            metavar="<auth-project-domain-id>",
                            default=os.environ.get("OS_PROJECT_DOMAIN_ID"),
                            help="defaults to env[OS_PROJECT_DOMAIN_ID]")

        parser.add_argument("--os-project-domain-name",
                            metavar="<auth-project-domain-name>",
                            default=os.environ.get("OS_PROJECT_DOMAIN_NAME"),
                            help="defaults to env[OS_PROJECT_DOMAIN_NAME]")

        parser.add_argument("--os-auth-url",
                            metavar="<auth-url>",
                            default=os.environ.get("OS_AUTH_URL"),
                            help="defaults to env[OS_AUTH_URL]")

        parser.add_argument("--bypass-url",
                            metavar="<bypass-url>",
                            dest="bypass_url",
                            help="use this API endpoint instead of the "
                                 "Service Catalog")

        parser.add_argument("--os-cacert",
                            metavar="<ca-certificate>",
                            default=os.environ.get("OS_CACERT", None),
                            help="Specify a CA bundle file to use in verifying"
                                 " a TLS (https) server certificate. Defaults "
                                 "to env[OS_CACERT]")

        subparser = parser.add_subparsers(help="commands", dest="command_name")
        commands = {}

        for entry in iter_entry_points(COMMANDS_ENTRY_POINT):
            command_class = entry.load()
            command = command_class()
            commands[entry.name] = command

        for command_name in sorted(commands.keys()):
            commands[command_name].configureParser(subparser)

        args = parser.parse_args(sys.argv[1:])

        os_username = args.os_username
        os_password = args.os_password
        os_user_domain_id = args.os_user_domain_id
        os_user_domain_name = args.os_user_domain_name
        os_project_name = args.os_project_name
        os_project_domain_id = args.os_project_domain_id
        os_project_domain_name = args.os_project_domain_name
        os_auth_url = args.os_auth_url
        os_cacert = args.os_cacert
        bypass_url = args.bypass_url
        command_name = args.command_name
        token = None

        if not os_username:
            raise ValueError("'os-username' not defined!")

        if not os_password:
            raise ValueError("'os-password' not defined!")

        if not os_project_name:
            raise ValueError("'os-project-name' not defined!")

        if not os_auth_url:
            raise ValueError("'os-auth-url' not defined!")

        if not os_user_domain_name:
            os_user_domain_name = "default"

        if not os_project_domain_name:
            os_project_domain_name = "default"

        client = keystone_v3.KeystoneClient(
            auth_url=os_auth_url,
            username=os_username,
            password=os_password,
            ca_cert=os_cacert,
            user_domain_id=os_user_domain_id,
            user_domain_name=os_user_domain_name,
            project_name=os_project_name,
            project_domain_id=os_project_domain_id,
            project_domain_name=os_project_domain_name)

        token = client.authenticate()

        if bypass_url:
            synergy_url = bypass_url
        else:
            synergy_endpoint = client.getEndpoint("synergy")
            synergy_url = synergy_endpoint["url"]

        if command_name not in commands:
            print("Command %r not found!" % command_name)

        commands[command_name].setToken(token)
        commands[command_name].execute(synergy_url, args)
    except KeyboardInterrupt:
        print("Shutting down synergyclient")
        sys.exit(1)
    except ConnectionError as ex:
        print("Failed to establish a new connection to %s" % synergy_url)
        sys.exit(1)
    except HTTPError as ex:
        if ex.response._content:
            print("%s" % ex.response._content)
        else:
            print("%s" % ex.message)
        sys.exit(1)
    except RequestException as ex:
        print("%s" % ex.response._content)
        sys.exit(1)
    except Exception as ex:
        print(ex)
        sys.exit(1)


if __name__ == "__main__":
    main()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from synergy.common.serializer import SynergyEncoder
from synergy.common.serializer import SynergyObject
from synergy.tests import base


class TestSerializer(base.TestCase):

    def setUp(self):
        super(TestSerializer, self).setUp()
        self.ser = SynergyObject()

    def test_set_get_Id(self):
        self.ser.setId('test_id')
        self.assertEqual('test_id', self.ser.getId())

    def test_set_get_Name(self):
        self.ser.setName('test_name')
        self.assertEqual('test_name', self.ser.getName())

    def test_set_get(self):
        self.ser.set('value', 'test_value')
        self.assertEqual('test_value', self.ser.get('value'))

    def test_Serialize(self):
        result = self.ser.serialize()
        self.assertEqual('1.0', result["synergy_object"]["version"])
        self.assertEqual("synergy", result["synergy_object"]["namespace"])

    def test_Deserialize(self):
        synObj = SynergyObject()
        synObj.setName('test_name')
        synObjSer = synObj.serialize()
        objInst = SynergyObject.deserialize(synObjSer)
        self.assertEqual('test_name', objInst.getName())


class TestSynergyEncoder(base.TestCase):

    def setUp(self):
        super(TestSynergyEncoder, self).setUp()
        self.se = SynergyEncoder()

    def test_SynergyEncoder(self):
        synObj = SynergyObject()
        synObjEnc = self.se.default(synObj)
        self.assertEqual('1.0', synObjEnc["synergy_object"]["version"])



# coding: utf-8
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Tests the (almost) abstract Manager class.

Since most of the functionalities of this class are present only in an
implementation of the class, we can just do very basic testing.
"""

from synergy.common.manager import Manager
from synergy.tests import base


class TestManager(base.TestCase):

    def setUp(self):
        super(TestManager, self).setUp()
        self.manager = Manager(name="dummy_manager")
        self.manager.setAutoStart(False)

    def test_get_name(self):
        self.assertEqual("dummy_manager", self.manager.getName())

    def test_get_managers_empty(self):
        self.assertEqual({}, self.manager.getManagers())

    def test_get_options(self):
        self.assertEqual([], self.manager.getOptions())

    def test_is_autostart(self):
        self.assertEqual(False, self.manager.isAutoStart())

    def test_set_autostart(self):
        self.manager.setAutoStart(True)
        self.assertEqual(True, self.manager.isAutoStart())

    def test_get_rate(self):
        self.assertEqual(-1, self.manager.getRate())

    def test_set_rate(self):
        self.manager.setRate(10)
        self.assertEqual(10, self.manager.getRate())

    def test_get_status(self):
        self.assertEqual("CREATED", self.manager.getStatus())

    def test_set_status(self):
        self.manager.setStatus("TEST")
        self.assertEqual("TEST", self.manager.getStatus())
